"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const run_1 = require("../base/run");
const conf_1 = require("../services/conf");
const docker_1 = require("../services/docker");
const logger_1 = require("../services/logger");
const validate_1 = require("../services/validate");
const util_1 = require("../util");
const { resolve } = path;
const { RdTypes, cwd, rdcConfName } = conf_1.default;
class Run extends run_1.default {
    get from() {
        if (conf_1.default.rdType === RdTypes.Application) {
            const { container } = conf_1.default.getAppConf();
            return container.name;
        }
        if (conf_1.default.rdType === RdTypes.Container) {
            const rdcConfPath = resolve(cwd, rdcConfName);
            const rdcConf = require(rdcConfPath);
            return rdcConf.extends || rdcConf.nodeVersion || 'node:latest';
        }
    }
    get workDir() {
        const { dockerWorkDirRoot } = conf_1.default;
        let name;
        if (conf_1.default.rdType === RdTypes.Application) {
            const { container } = conf_1.default.getAppConf();
            name = container.name.split(':')[0];
        }
        if (conf_1.default.rdType === RdTypes.Container) {
            const rdcConfPath = resolve(cwd, rdcConfName);
            const rdcConf = require(rdcConfPath);
            const { tag } = rdcConf.docker;
            name = tag.split(':')[0];
        }
        return `${dockerWorkDirRoot}/${name}`;
    }
    get tag() {
        if (conf_1.default.rdType === RdTypes.Container) {
            const rdcConf = require(`${conf_1.default.cwd}/${conf_1.default.rdcConfName}`);
            return rdcConf.docker.tag || null;
        }
    }
    get ports() {
        if (conf_1.default.rdType === RdTypes.Application) {
            const { docker } = conf_1.default.getAppConf();
            return docker.ports || [];
        }
        if (conf_1.default.rdType === RdTypes.Container) {
            const rdcConf = require(`${conf_1.default.cwd}/${conf_1.default.rdcConfName}`);
            return rdcConf.docker.ports || [];
        }
    }
    async preInit() {
        await super.preInit();
        await docker_1.default.checkEnv();
        if (conf_1.default.rdType === RdTypes.Container) {
            await validate_1.validateRdc();
        }
        if (conf_1.default.rdType === RdTypes.Application) {
            await validate_1.validateRda();
        }
    }
    async preRun() {
        await docker_1.default.genDockerFile(this.workDir, this.from, conf_1.default.dockerTmpDir);
        await docker_1.default.genDockerCompose(this.workDir, this.cmd, this.ports, this.watch, this.tag, conf_1.default.dockerTmpDir);
    }
    async run() {
        if (!await docker_1.default.imageExist(this.tag)) {
            logger_1.spinner.start('Building image start');
            await util_1.default.asyncExec(`cd ${conf_1.default.dockerTmpDir} && docker-compose build`);
            logger_1.spinner.stop();
        }
        if (this.watch) {
            await util_1.default.asyncSpawn('docker-compose', ['run', '--rm', 'rde', 'rde', 'docker:run', this.cmd, '--watch'], {
                cwd: conf_1.default.dockerTmpDir,
            });
        }
        else {
            await util_1.default.asyncSpawn('docker-compose', ['run', '--rm', 'rde', 'rde', 'docker:run', this.cmd], {
                cwd: conf_1.default.dockerTmpDir,
            });
        }
    }
}
Run.description = 'run scripts provided by container';
Run.examples = [
    '$ rde run <cmd>',
];
Run.flags = Object.assign({}, run_1.default.flags);
Run.args = [{
        name: 'cmd',
        required: true,
        description: 'scripts provided by container',
    }];
exports.default = Run;
