"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const extend = require("deep-extend");
const path = require("path");
const util_1 = require("../util");
const { resolve } = path;
const appConfName = 'rda.config.js';
const rdcConfName = 'rdc.config.js';
const rdsConfName = 'rds.config.js';
let rdType = '';
const conf = {
    get cwd() {
        return process.cwd();
    },
    get docsDir() {
        return '_docs';
    },
    get docsPagesDir() {
        return '.docs';
    },
    get appConfPath() {
        return resolve(conf.cwd, appConfName);
    },
    get rdcConfPath() {
        return resolve(conf.cwd, rdcConfName);
    },
    get dockerWorkDirRoot() {
        return '/usr/rde';
    },
    get tmpDir() {
        return '/tmp/rde_rdc_tmp';
    },
    get runtimeDir() {
        return resolve(conf.cwd, `.${conf.cliName}`);
    },
    get dockerTmpDir() {
        return '.docker';
    },
    get cliName() { return 'rde'; },
    get homepage() { return 'https://github.com/kaola-fed/RDE'; },
    get appConfName() { return appConfName; },
    get rdcConfName() { return rdcConfName; },
    get rdsConfName() { return rdsConfName; },
    get frameworks() {
        return {
            vue: {
                rdcStarter: 'nupthale/rdc-vue-starter',
                cdn: [],
            },
            react: {
                rdcStarter: 'rdc-react-starter',
                cdn: [
                    'https://unpkg.com/react/umd/react.production.min.js',
                    'https://unpkg.com/react-dom/umd/react-dom.production.min.js',
                ],
            },
            angular: {
                rdcStarter: 'rdc-angular-starter',
                cdn: ['https://ajax.googleapis.com/ajax/libs/angularjs/1.4.5/angular.min.js'],
            },
        };
    },
    get RdTypes() {
        return {
            Application: 'Application',
            Container: 'Container',
            Suite: 'Suite',
        };
    },
    get rdType() {
        return rdType;
    },
    set rdType(type) {
        rdType = type;
    },
    getAppConf() {
        return util_1.default.ensureRequire(conf.appConfPath);
    },
    getRdcDir(rdc) {
        // rdeName is the dir contain the rdc codes
        const rdcName = rdc.split(':')[0];
        return `../${rdcName}`;
    },
    getRdcChain(nodeDir, chain = []) {
        chain.push(nodeDir);
        const rdcConfPath = resolve(nodeDir, conf.rdcConfName);
        const { extend } = require(rdcConfPath);
        if (!extend) {
            return chain.reverse();
        }
        return conf.getRdcChain(`${conf.getRdcDir(extend)}`, chain);
    },
    getRdcConf(nodeDir) {
        const chain = conf.getRdcChain(nodeDir);
        return conf.getRdcConfFromChain(chain);
    },
    getRdcConfFromChain(chain) {
        let merged;
        for (let node of chain) {
            // node is an array like ['.' '../rdc1', '../rdcScope/rdc2']
            const nodeConf = require(resolve(conf.cwd, node, conf.rdcConfName));
            merged = extend({}, nodeConf, merged || {});
        }
        return merged;
    },
};
exports.default = conf;
